import { world, system } from '@minecraft/server'; 
import Logger from '../utils/Logger.js';

class PlayerStateManager {
    constructor() {
        this.states = new Map();
        this.initializeEventListeners();
    }

    initializeEventListeners() {
        world.afterEvents.playerSpawn.subscribe((event) => {
            const playerName = event.player.name;
            if (!this.states.has(playerName)) {
                this.initializePlayer(playerName);
            }
        });

        world.afterEvents.playerLeave.subscribe((event) => {
            this.clearPlayerStates(event.playerName);
        });
    }

    getState(playerName) {
        if (!this.states.has(playerName)) {
            this.initializePlayer(playerName);
        }
        return this.states.get(playerName);
    }

    initializePlayer(playerName) {
        const initialState = {
            roll: {
                isInAir: false,
                isFalling: false,
                fallStartY: 0,
                rollInputReceived: false,
                isRolling: false,
                rollStartTime: 0,
                lastY: 0,
                validFallHeight: false,
                lastRollTime: null,
                lastCooldownMessage: 0,
                lastUpdateTime: 0,
                inLiquid: false,
                wasGliding: false,
                rollInputTime: null
            }
        };

        this.states.set(playerName, initialState);
        Logger.debug(`Initialized state for player ${playerName}`, "SYSTEM");
    }

    setRollState(playerName, currentTime) {
        const state = this.getState(playerName);
        state.roll.isRolling = true;
        state.roll.rollStartTime = currentTime;
        Logger.debug(`Player ${playerName} started rolling`, "ROLL");
    }

    clearRollState(playerName, currentTime) {
        const state = this.getState(playerName);
        state.roll.isRolling = false;
        state.roll.lastRollTime = currentTime;
        Logger.debug(`Player ${playerName} roll ended`, "ROLL");
    }

    clearPlayerStates(playerName) {
        if (this.states.has(playerName)) {
            this.states.delete(playerName);
            Logger.info(`Cleared all states for player ${playerName}`, "SYSTEM");
        }
    }
}

export default new PlayerStateManager();